from enum import Enum
class OsType(Enum):
    GENERIC_WINDOWS = "GENERIC_WINDOWS"
    GENERIC_WINDOWS_SERVER = "GENERIC_WINDOWS_SERVER"
    GENERIC_LINUX = "GENERIC_LINUX"
    WINDOWS_TPM = "WINDOWS_TPM"
